#pragma once
#include "ros_bridge/include/MessageBaseClass.h"
#include "ros_bridge/include/MessageTag.h"
#include "ros_bridge/include/TypeFactory.h"
#include "ros_bridge/include/JsonFactory.h"

#include <memory>
#include "rapidjson/include/rapidjson/document.h"

namespace ROSBridge {

class CasePacker
{
    typedef MessageTag Tag;
    typedef rapidjson::Document JsonDoc;
    typedef std::unique_ptr<JsonDoc> JsonDocUPtr;
public:
    CasePacker() = delete;
    CasePacker(TypeFactory *typeFactory, JsonFactory *jsonFactory);

    template<class T>
    JsonDoc *pack(const T &msg, const std::string &topic) const
    {
        JsonDoc *docPt(_jsonFactory->create(msg));
        std::string messageType = T::Group::messageType();
        addTag(*docPt, topic, messageType.c_str());
        return docPt;
    }

    template<class T>
    bool unpack(JsonDoc &doc, T &msg) const {
        removeTag(doc);
        return _typeFactory->create(doc, msg);
    }


    bool getTag(const JsonDoc &doc, Tag &tag) const;
    void addTag        (JsonDoc &doc,
                         const std::string &topic,
                         const std::string &messageType) const;
    void addTag        (JsonDoc &doc, const Tag &tag) const;
    void removeTag     (JsonDoc &doc) const;
    bool getTopic      (const JsonDoc &doc, std::string &topic) const;
    bool getMessageType(const JsonDoc &doc, std::string &messageType) const;

    static const char* topicKey;
    static const char* messageTypeKey;

private:
    TypeFactory *_typeFactory;
    JsonFactory *_jsonFactory;
};
}

